@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 """
 Entry point for the ledfx controller. To run this script for development
 purposes use:
@@ -17,13 +16,25 @@ For non-development purposes run:
 """
 
 import argparse
+import importlib
 import logging
+import os
 import subprocess
 import sys
-import warnings
 from logging.handlers import RotatingFileHandler
 
-from pyupdater.client import Client
+try:
+    import psutil
+
+    have_psutil = True
+except ImportError:
+    have_psutil = False
+try:
+    from pyupdater.client import Client
+
+    have_updater = True
+except ImportError:
+    have_updater = False
 
 import ledfx.config as config_helpers
 from ledfx.consts import (
@@ -70,8 +81,8 @@ def reset_logging():
                 logger.removeHandler(handler)
 
 
-def setup_logging(loglevel):
-    # Create a custom logging level to display pyupdater progress
+def setup_logging(loglevel, config_dir):
+    # Create a custom logging level to virtual pyupdater progress
     reset_logging()
 
     console_loglevel = loglevel or logging.WARNING
@@ -83,19 +94,15 @@ def setup_logging(loglevel):
     root_logger = logging.getLogger()
 
     file_handler = RotatingFileHandler(
-        config_helpers.get_log_file_location(),
+        config_helpers.get_log_file_location(config_dir),
         mode="a",  # append
         maxBytes=0.5 * 1000 * 1000,  # 512kB
         encoding="utf8",
         backupCount=5,  # once it hits 2.5MB total, start removing logs.
     )
     file_handler.setLevel(file_loglevel)  # set loglevel
-    file_formatter = logging.Formatter(
-        file_logformat
-    )  # a simple log file format
-    file_handler.setFormatter(
-        file_formatter
-    )  # tell the file_handler to use this format
+    file_formatter = logging.Formatter(file_logformat)
+    file_handler.setFormatter(file_formatter)
 
     console_handler = logging.StreamHandler()
     console_handler.setLevel(console_loglevel)  # set loglevel
@@ -117,7 +124,7 @@ def setup_logging(loglevel):
     logging.getLogger("sacn").setLevel(logging.WARNING)
     logging.getLogger("aiohttp.access").setLevel(logging.WARNING)
     logging.getLogger("pyupdater").setLevel(logging.WARNING)
-    logging.getLogger("zeroconf").setLevel(logging.DEBUG)
+    logging.getLogger("zeroconf").setLevel(logging.WARNING)
 
     global _LOGGER
     _LOGGER = logging.getLogger(__name__)
@@ -131,7 +138,7 @@ def parse_args():
     parser.add_argument(
         "--version",
         action="version",
-        version="ledfx {ver}".format(ver=PROJECT_VERSION),
+        version=f"ledfx {PROJECT_VERSION}",
     )
     parser.add_argument(
         "-c",
@@ -167,7 +174,15 @@ def parse_args():
         "-p",
         "--port",
         dest="port",
-        help="Web interface port",
+        help="Web interface port (HTTP)",
+        default=None,
+        type=int,
+    )
+    parser.add_argument(
+        "-p_s",
+        "--port_secure",
+        dest="port_s",
+        help="Web interface port (HTTPS)",
         default=None,
         type=int,
     )
@@ -178,6 +193,12 @@ def parse_args():
         default=None,
         type=str,
     )
+    parser.add_argument(
+        "--tray",
+        dest="tray",
+        action="store_true",
+        help="Hide LedFx console to the system tray",
+    )
     parser.add_argument(
         "--offline",
         dest="offline_mode",
@@ -199,6 +220,9 @@ def installed_via_pip():
     Returns:
         boolean
     """
+    pip_spec = importlib.util.find_spec("pip")
+    if pip_spec is None:
+        return False
     pip_package_command = subprocess.check_output(
         [sys.executable, "-m", "pip", "freeze"]
     )
@@ -211,11 +235,26 @@ def installed_via_pip():
         return False
 
 
-def update_ledfx():
-
+def update_ledfx(icon=None):
     # initialize & refresh in one update, check client
 
-    class ClientConfig(object):
+    def notify(msg):
+        if icon and icon.HAS_NOTIFICATION:
+            icon.remove_notification()
+            icon.notify(msg)
+        _LOGGER.log(PYUPDATERLOGLEVEL, msg)
+
+    def log_status_info(info):
+        total = info.get("total")
+        downloaded = info.get("downloaded")
+        percent_complete = info.get("percent_complete")
+        time = info.get("time")
+        _LOGGER.log(
+            PYUPDATERLOGLEVEL,
+            f"{downloaded} of {total} [{percent_complete} complete, {time} remaining]",
+        )
+
+    class ClientConfig:
         PUBLIC_KEY = "Txce3TE9BUixsBtqzDba6V5vBYltt/0pw5oKL8ueCDg"
         APP_NAME = PROJECT_NAME
         COMPANY_NAME = "LedFx Developers"
@@ -229,20 +268,21 @@ def update_ledfx():
     # If an update is found, an update object will be returned
     # If no updates are available, None will be returned
     ledfx_update = client.update_check(PROJECT_NAME, PROJECT_VERSION)
+
     # Download the update
     if ledfx_update is not None:
+        client.add_progress_hook(log_status_info)
         _LOGGER.log(PYUPDATERLOGLEVEL, "Update found!")
-        _LOGGER.log(PYUPDATERLOGLEVEL, "Downloading update, please wait...")
+        notify(
+            "Downloading update, please wait... LedFx will restart when complete."
+        )
         ledfx_update.download()
         # Install and restart
         if ledfx_update.is_downloaded():
-            _LOGGER.log(
-                PYUPDATERLOGLEVEL,
-                "Update downloaded, extracting and restarting...",
-            )
+            notify("Download complete. Restarting LedFx...")
             ledfx_update.extract_restart()
         else:
-            _LOGGER.error("Unable to download update.")
+            notify("Unable to download update.")
     else:
         # No Updates, into main we go
         _LOGGER.log(
@@ -255,34 +295,100 @@ def main():
     """Main entry point allowing external calls"""
     args = parse_args()
     config_helpers.ensure_config_directory(args.config)
-    setup_logging(args.loglevel)
+    setup_logging(args.loglevel, config_dir=args.config)
     config_helpers.load_logger()
 
+    # Set some process priority optimisations
+    if have_psutil:
+        p = psutil.Process(os.getpid())
+
+        if psutil.WINDOWS:
+            try:
+                p.nice(psutil.HIGH_PRIORITY_CLASS)
+            except psutil.Error:
+                _LOGGER.info(
+                    "Unable to set priority, please run as Administrator if you are experiencing frame rate issues"
+                )
+            # p.ionice(psutil.IOPRIO_HIGH)
+        elif psutil.LINUX:
+            try:
+                p.nice(15)
+                p.ionice(psutil.IOPRIO_CLASS_RT, value=7)
+            except psutil.Error:
+                _LOGGER.info(
+                    "Unable to set priority, please run as root or sudo if you are experiencing frame rate issues",
+                )
+        else:
+            p.nice(15)
+
+    if (
+        not (currently_frozen() or installed_via_pip())
+        and args.offline_mode is False
+    ):
+        import ledfx.sentry_config  # noqa: F401
+
     if args.sentry_test:
         """This will crash LedFx and submit a Sentry error if Sentry is configured"""
         _LOGGER.warning("Steering LedFx into a brick wall")
         div_by_zero = 1 / 0
 
-    if args.offline_mode is False and currently_frozen():
+    if args.tray or currently_frozen():
+        # If pystray is imported on a device that can't display it, it explodes. Catch it
+        try:
+            import pystray
+        except Exception as Error:
+            msg = f"Error: Unable to virtual tray icon. Shutting down. Error: {Error}"
+            _LOGGER.critical(msg)
+            raise Exception(msg)
+            sys.exit(0)
 
-        warnings.filterwarnings("ignore", category=DeprecationWarning)
-        update_ledfx()
+        from PIL import Image
 
-    if args.offline_mode:
-        _LOGGER.warning(
-            "Offline Mode Enabled - Please check for updates regularly."
+        if currently_frozen():
+            current_directory = os.path.dirname(__file__)
+            icon_location = os.path.join(current_directory, "tray.png")
+        else:
+            current_directory = os.path.dirname(__file__)
+            icon_location = os.path.join(
+                current_directory, "..", "icons/" "tray.png"
+            )
+        icon = pystray.Icon(
+            "LedFx", icon=Image.open(icon_location), title="LedFx"
         )
-    if args.offline_mode is False:
-        import ledfx.sentry_config  # noqa: F401
+        icon.visible = True
+    else:
+        icon = None
+    # icon = None
 
-    if not currently_frozen() and installed_via_pip():
-        warnings.filterwarnings("ignore", category=DeprecationWarning)
+    # if have_updater and not args.offline_mode and currently_frozen():
+    #     update_ledfx(icon)
 
-    _LOGGER.info("LedFx Core is initializing")
+    if icon:
+        icon.run(setup=entry_point)
+    else:
+        entry_point()
 
-    ledfx = LedFxCore(config_dir=args.config, host=args.host, port=args.port)
 
-    ledfx.start(open_ui=args.open_ui)
+def entry_point(icon=None):
+    # have to re-parse args here :/ no way to pass them through pysicon's setup
+    args = parse_args()
+
+    exit_code = 4
+    while exit_code == 4:
+        _LOGGER.info("LedFx Core is initializing")
+
+        ledfx = LedFxCore(
+            config_dir=args.config,
+            host=args.host,
+            port=args.port,
+            port_s=args.port_s,
+            icon=icon,
+        )
+
+        exit_code = ledfx.start(open_ui=args.open_ui)
+
+    if icon:
+        icon.stop()
 
 
 if __name__ == "__main__":
