@@ -3,7 +3,9 @@ import collections
 
 import voluptuous as vol
 
-from ledfx.utils import generate_title
+from ledfx.config import _default_wled_settings
+from ledfx.effects.audio import AudioInputSource
+from ledfx.utils import AVAILABLE_FPS, generate_title
 
 TYPES_MAP = {
     int: "integer",
@@ -11,6 +13,26 @@ TYPES_MAP = {
     float: "number",
     bool: "boolean",
     list: "array",
+    dict: "dict",
+}
+
+PERMITTED_KEYS = {
+    "audio": ("min_volume", "audio_device", "delay_ms"),
+    "melbanks": (
+        "max_frequencies",
+        "min_frequency",
+    ),
+    "wled_preferences": tuple(_default_wled_settings.keys()),
+    "core": (
+        "host",
+        "port",
+        "port_s",
+        "dev_mode",
+        "scan_on_startup",
+        "visualisation_fps",
+        "visualisation_maxlen",
+        "global_transitions",
+    ),
 }
 
 
@@ -76,7 +98,31 @@ def convertToJsonSchema(schema):
 
         return val
 
-    if isinstance(schema, vol.All):
+    if (
+        callable(schema)
+        and getattr(schema, "__name__", None) == "fps_validator"
+    ):
+        return {"type": "int", "enum": list(AVAILABLE_FPS)}
+
+    elif (
+        callable(schema)
+        and getattr(schema, "__name__", None) == "device_index_validator"
+    ):
+        return {"type": "string", "enum": AudioInputSource.input_devices()}
+
+    elif (
+        callable(schema)
+        and getattr(schema, "__name__", None) == "validate_color"
+    ):
+        return {"type": "color", "gradient": False}
+
+    elif (
+        callable(schema)
+        and getattr(schema, "__name__", None) == "validate_gradient"
+    ):
+        return {"type": "color", "gradient": True}
+
+    elif isinstance(schema, vol.All):
         val = {}
         for validator in schema.validators:
             val.update(convertToJsonSchema(validator))
@@ -105,7 +151,10 @@ def convertToJsonSchema(schema):
         }
 
     elif isinstance(schema, vol.In):
-        return {"type": "string", "enum": list(schema.container)}
+        if isinstance(schema.container, dict):
+            return {"type": "string", "enum": dict(schema.container)}
+        else:
+            return {"type": "string", "enum": list(schema.container)}
         # val = {'type': 'string', 'enum': dict()}
         # for item in schema.container:
         #     val['enum'][item] = item
@@ -114,7 +163,16 @@ def convertToJsonSchema(schema):
     elif isinstance(schema, vol.Coerce):
         schema = schema.type
 
+    elif isinstance(schema, list):
+        val = {
+            "type": "list",
+            "validators": list(
+                convertToJsonSchema(validator) for validator in schema
+            ),
+        }
+        return val
+
     if schema in TYPES_MAP:
         return {"type": TYPES_MAP[schema]}
 
-    raise ValueError("Unable to convert schema: {}".format(schema))
+    raise ValueError(f"Unable to convert schema: {schema}")
