@@ -1,5 +1,6 @@
 import colorsys
 import logging
+import threading
 
 # from ledfx.effects.audio import FREQUENCY_RANGES
 from functools import lru_cache
@@ -7,12 +8,38 @@ from functools import lru_cache
 import numpy as np
 import voluptuous as vol
 
-from ledfx.color import COLORS
+from ledfx.color import parse_color, validate_color
 from ledfx.utils import BaseRegistry, RegistryLoader
 
 _LOGGER = logging.getLogger(__name__)
 
 
+class DummyEffect:
+
+    config = vol.Schema({})
+    _active = True
+    is_active = _active
+    NAME = name = ""
+
+    def __init__(self, pixel_count):
+        self.pixels = np.zeros((pixel_count, 3))
+
+    def _render(self):
+        pass
+
+    def render(self):
+        pass
+
+    def get_pixels(self):
+        return self.pixels
+
+    def activate(self):
+        pass
+
+    def deactivate(self):
+        pass
+
+
 def mix_colors(color_1, color_2, ratio):
     if np.array_equal(color_2, []):
         return (
@@ -28,12 +55,6 @@ def mix_colors(color_1, color_2, ratio):
         )
 
 
-def fill_solid(pixels, color):
-    pixels[
-        :,
-    ] = color
-
-
 def fill_rainbow(pixels, initial_hue, delta_hue):
     hue = initial_hue
     sat = 0.95
@@ -46,22 +67,6 @@ def fill_rainbow(pixels, initial_hue, delta_hue):
     return pixels
 
 
-def mirror_pixels(pixels):
-    # TODO: Figure out some better logic here. Needs to reduce the signal
-    # and reflect across the middle. The prior logic was broken for
-    # non-uniform effects.
-    mirror_shape = (np.shape(pixels)[0], 2, np.shape(pixels)[1])
-    return (
-        np.append(pixels[::-1], pixels, axis=0)
-        .reshape(mirror_shape)
-        .mean(axis=1)
-    )
-
-
-def flip_pixels(pixels):
-    return np.flipud(pixels)
-
-
 def blur_pixels(pixels, sigma):
     rgb_array = pixels.T
     rgb_array[0] = smooth(rgb_array[0], sigma)
@@ -70,13 +75,8 @@ def blur_pixels(pixels, sigma):
     return rgb_array.T
 
 
-def brightness_pixels(pixels, brightness):
-    pixels = np.multiply(pixels, brightness, out=pixels, casting="unsafe")
-    return pixels
-
-
 @lru_cache(maxsize=32)
-def _gaussian_kernel1d(sigma, order, radius):
+def _gaussian_kernel1d(sigma, order, array_len):
     """
     Produces a 1D Gaussian or Gaussian-derivative filter kernel as a numpy array.
 
@@ -88,6 +88,14 @@ def _gaussian_kernel1d(sigma, order, radius):
     Returns:
         Array of length (2*radius+1) containing the filter kernel.
     """
+
+    # Choose a radius for the filter kernel large enough to include all significant elements. Using
+    # a radius of 4 standard deviations (rounded to int) will only truncate tail values that are of
+    # the order of 1e-5 or smaller. For very small sigma values, just use a minimal radius.
+    radius = max(1, int(round(4.0 * sigma)))
+    radius = min(int((array_len - 1) / 2), radius)
+    radius = max(radius, 1)
+
     if order < 0:
         raise ValueError("Order must non-negative")
     if not (isinstance(radius, int) or radius.is_integer()) or radius <= 0:
@@ -111,6 +119,23 @@ def _gaussian_kernel1d(sigma, order, radius):
     return phi_x
 
 
+def fast_blur_pixels(pixels, sigma):
+    if len(pixels) == 0:
+        raise ValueError("Cannot smooth an empty array")
+    kernel = _gaussian_kernel1d(sigma, 0, len(pixels))
+    pixels[:, 0] = np.convolve(pixels[:, 0], kernel, mode="same")
+    pixels[:, 1] = np.convolve(pixels[:, 1], kernel, mode="same")
+    pixels[:, 2] = np.convolve(pixels[:, 2], kernel, mode="same")
+    return pixels
+
+
+def fast_blur_array(array, sigma):
+    if len(array) == 0:
+        raise ValueError("Cannot smooth an empty array")
+    kernel = _gaussian_kernel1d(sigma, 0, len(array))
+    return np.convolve(array, kernel, mode="same")
+
+
 def smooth(x, sigma):
     """
     Smooths a 1D array via a Gaussian filter.
@@ -172,10 +197,9 @@ class Effect(BaseRegistry):
     """
 
     NAME = ""
-    _pixels = None
-    _dirty = False
     _config = None
     _active = False
+    _virtual = None
 
     # Basic effect properties that can be applied to all effects
     CONFIG_SCHEMA = vol.Schema(
@@ -200,42 +224,65 @@ class Effect(BaseRegistry):
             ): vol.All(vol.Coerce(float), vol.Range(min=0.0, max=1.0)),
             vol.Optional(
                 "background_color",
-                description="Apply a background colour",
-                default="black",
-            ): vol.In(list(COLORS.keys())),
+                description="Apply a background color",
+                default="#000000",
+            ): validate_color,
+            vol.Optional(
+                "background_brightness",
+                description="Brightness of the background color",
+                default=1.0,
+            ): vol.All(vol.Coerce(float), vol.Range(min=0.0, max=1.0)),
         }
     )
 
     def __init__(self, ledfx, config):
         self._ledfx = ledfx
-        self._dirty_callback = None
+        self._config = {}
         self.update_config(config)
+        self.lock = threading.Lock()
 
     def __del__(self):
         if self._active:
             self.deactivate()
 
-    def activate(self, pixel_count):
+    def activate(self, virtual):
         """Attaches an output channel to the effect"""
-        self._pixels = np.zeros((pixel_count, 3))
-        self._active = True
+        self._virtual = virtual
+        self.pixels = np.zeros((virtual.pixel_count, 3))
+        # Iterate all the base classes and check to see if the base
+        # class has an on_activate method. If so, call it
+        valid_classes = list(type(self).__bases__)
+        valid_classes.append(type(self))
+        for base in valid_classes:
+            if hasattr(base, "on_activate"):
+                base.on_activate(self, virtual.pixel_count)
 
+        self._active = True
         _LOGGER.info(f"Effect {self.NAME} activated.")
 
     def deactivate(self):
         """Detaches an output channel from the effect"""
-        self._pixels = None
+        self.pixels = None
         self._active = False
 
         _LOGGER.info(f"Effect {self.NAME} deactivated.")
 
     def update_config(self, config):
         # TODO: Sync locks to ensure everything is thread safe
-        validated_config = type(self).schema()(config)
-        self._config = validated_config
 
-        self._bg_color = np.array(
-            COLORS[self._config["background_color"]], dtype=float
+        validated_config = type(self).schema()(config)
+        prior_config = self._config
+
+        if self._config != {}:
+
+            self._config = {**prior_config, **config}
+        else:
+            self._config = validated_config
+        self.configured_blur = self._config["blur"]
+
+        self._bg_color = (
+            np.array(parse_color(self._config["background_color"]))
+            * self._config["background_brightness"]
         )
 
         def inherited(cls, method):
@@ -251,7 +298,7 @@ class Effect(BaseRegistry):
             if base.config_updated != super(base, base).config_updated:
                 base.config_updated(self, self._config)
 
-        _LOGGER.info(
+        _LOGGER.debug(
             f"Effect {self.NAME} config updated to {validated_config}."
         )
 
@@ -264,69 +311,58 @@ class Effect(BaseRegistry):
         complex properties off the configuration, otherwise the config
         should just be referenced in the effect's loop directly
         """
-        self.configured_blur = self._config["blur"]
         pass
 
+    def _render(self):
+        self.lock.acquire()
+        self.render()
+        self.lock.release()
+
+    def render(self):
+        """
+        To be implemented by child effect
+        Must act on self.pixels, setting the values of it
+        The effect can use self.pixels to see the previous effect
+        frame if it wants to use it for something
+        """
+        pass
+
+    def get_pixels(self):
+        if not hasattr(self, "pixels"):
+            return
+        pixels = np.copy(self.pixels)
+        # Apply some of the base output filters if necessary
+        if self._config["flip"]:
+            pixels = np.flipud(pixels)
+        if self._config["mirror"]:
+            pixels = np.concatenate(
+                (pixels[-1 + len(pixels) % -2 :: -2], pixels[::2])
+            )
+        if self._config["background_color"]:
+            # TODO: colors in future should have an alpha value, which would work nicely to apply to dim the background color
+            # for now, just set it a bit less bright.
+            # pixels += self._bg_color * 0.5
+            pixels += self._bg_color
+        if self._config["brightness"] is not None:
+            np.multiply(
+                pixels,
+                self._config["brightness"],
+                out=pixels,
+                casting="unsafe",
+            )
+        # If the configured blur is greater than 0 we need to blur it
+        if self.configured_blur != 0.0:
+            kernel = _gaussian_kernel1d(self.configured_blur, 0, len(pixels))
+            pixels[:, 0] = np.convolve(pixels[:, 0], kernel, mode="same")
+            pixels[:, 1] = np.convolve(pixels[:, 1], kernel, mode="same")
+            pixels[:, 2] = np.convolve(pixels[:, 2], kernel, mode="same")
+        return pixels
+
     @property
     def is_active(self):
         """Return if the effect is currently active"""
         return self._active
 
-    def get_pixels(self):
-        return self.pixels
-
-    @property
-    def pixels(self):
-        """Returns the pixels for the channel"""
-        if not self._active:
-            raise Exception(
-                "Attempting to access pixels before effect is active"
-            )
-
-        return np.copy(self._pixels)
-
-    @pixels.setter
-    def pixels(self, pixels):
-        """Sets the pixels for the channel"""
-        if not self._active:
-            _LOGGER.warning(
-                "Attempting to set pixels before effect is active. Dropping."
-            )
-            return
-
-        if isinstance(pixels, tuple):
-            self._pixels = np.copy(pixels)
-        elif isinstance(pixels, np.ndarray):
-
-            # Apply some of the base output filters if necessary
-            if self._config["flip"]:
-                pixels = flip_pixels(pixels)
-            if self._config["mirror"]:
-                pixels = mirror_pixels(pixels)
-            if self._config["background_color"]:
-                # TODO: colours in future should have an alpha value, which would work nicely to apply to dim the background colour
-                # for now, just set it a bit less bright.
-                bg_brightness = np.max(pixels, axis=1)
-                bg_brightness = (255 - bg_brightness) / 510
-                _bg_color_array = np.tile(self._bg_color, (len(pixels), 1))
-                pixels += np.multiply(_bg_color_array.T, bg_brightness).T
-            if self._config["brightness"] is not None:
-                pixels = brightness_pixels(pixels, self._config["brightness"])
-            # If the configured blur is greater than 0 we need to blur it
-            if self.configured_blur != 0.0:
-                pixels = blur_pixels(pixels=pixels, sigma=self.configured_blur)
-            self._pixels = np.copy(pixels)
-        else:
-            raise TypeError()
-
-        self._dirty = True
-
-        if self._dirty_callback:
-            self._dirty_callback()
-
-    def setDirtyCallback(self, callback):
-        self._dirty_callback = callback
-
     @property
     def pixel_count(self):
         """Returns the number of pixels for the channel"""
