@@ -7,10 +7,11 @@ from ledfx.effects.temporal import TemporalEffect
 
 class FadeEffect(TemporalEffect, GradientEffect):
     """
-    Fades through the colours of a gradient
+    Fades through the colors of a gradient
     """
 
     NAME = "Fade"
+    CATEGORY = "Non-Reactive"
 
     CONFIG_SCHEMA = vol.Schema(
         {
@@ -26,6 +27,9 @@ class FadeEffect(TemporalEffect, GradientEffect):
         self.idx = 0
         self.forward = True
 
+    def on_activate(self, pixel_count):
+        pass
+
     def effect_loop(self):
         self.idx += 0.0015
         if self.idx > 1:
