@@ -7,6 +7,8 @@ from ledfx.effects.temporal import TemporalEffect
 class RainbowEffect(TemporalEffect):
 
     NAME = "Rainbow"
+    CATEGORY = "Non-Reactive"
+
     CONFIG_SCHEMA = vol.Schema(
         {
             vol.Optional(
@@ -19,6 +21,9 @@ class RainbowEffect(TemporalEffect):
 
     _hue = 0.1
 
+    def on_activate(self, pixel_count):
+        pass
+
     def effect_loop(self):
         hue_delta = self._config["frequency"] / self.pixel_count
         self.pixels = fill_rainbow(self.pixels, self._hue, hue_delta)
